@@ -29,6 +29,7 @@
  */
 
 #include "librc.h"
+#include "einfo.h"
 
 bool
 rc_yesno(const char *value)
@@ -127,6 +128,53 @@ rc_getline(char **line, size_t *len, FILE *fp)
 }
 librc_hidden_def(rc_getline)
 
+#ifdef __linux__
+char *
+rc_proc_getent(const char *ent)
+{
+	FILE *fp;
+	char *proc, *p, *value = NULL;
+	size_t i, len;
+
+	if (!exists("/proc/cmdline"))
+		return NULL;
+
+	if (!(fp = fopen("/proc/cmdline", "r"))) {
+		eerror("failed to open `/proc/cmdline': %s", strerror(errno));
+		return NULL;
+	}
+
+	proc = NULL;
+	i = 0;
+	if (rc_getline(&proc, &i, fp) == -1 || proc == NULL)
+		eerror("rc_getline: %s", strerror(errno));
+
+	if (proc != NULL) {
+		len = strlen(ent);
+
+		while ((p = strsep(&proc, " "))) {
+			if (strncmp(ent, p, len) == 0 && (p[len] == '\0' || p[len] == ' ' || p[len] == '=')) {
+				p += len;
+
+				if (*p == '=')
+					p++;
+
+				value = xstrdup(p);
+			}
+		}
+	}
+
+	if (!value)
+		errno = ENOENT;
+
+	fclose(fp);
+	free(proc);
+
+	return value;
+}
+librc_hidden_def(rc_proc_getent)
+#endif
+
 RC_STRINGLIST *
 rc_config_list(const char *file)
 {
@@ -166,6 +214,64 @@ rc_config_list(const char *file)
 }
 librc_hidden_def(rc_config_list)
 
+/*
+ * Override some specific rc.conf options on the kernel command line
+ */
+#ifdef __linux__
+static RC_STRINGLIST *rc_config_override(RC_STRINGLIST *config)
+{
+	RC_STRINGLIST *overrides;
+	RC_STRING *cline, *override, *config_np;
+	char *tmp = NULL;
+	char *value = NULL;
+	size_t varlen = 0;
+	size_t len = 0;
+
+	overrides = rc_stringlist_new();
+
+	/* A list of variables which may be overridden on the kernel command line */
+	rc_stringlist_add(overrides, "rc_parallel");
+
+	TAILQ_FOREACH(override, overrides, entries) {
+		varlen = strlen(override->value);
+		value = rc_proc_getent(override->value);
+
+		/* No need to continue if there's nothing to override */
+		if (!value) {
+			free(value);
+			continue;
+		}
+
+		if (value != NULL) {
+			len = varlen + strlen(value) + 2;
+			tmp = xmalloc(sizeof(char) * len);
+			snprintf(tmp, len, "%s=%s", override->value, value);
+		}
+
+		/*
+		 * Whenever necessary remove the old config entry first to prevent
+		 * duplicates
+		 */
+		TAILQ_FOREACH_SAFE(cline, config, entries, config_np) {
+			if (strncmp(override->value, cline->value, varlen) == 0
+				&& cline->value[varlen] == '=') {
+				rc_stringlist_delete(config, cline->value);
+				break;
+			}
+		}
+
+		/* Add the option (var/value) to the current config */
+		rc_stringlist_add(config, tmp);
+
+		free(tmp);
+		free(value);
+	}
+
+	rc_stringlist_free(overrides);
+	return config;
+}
+#endif
+
 RC_STRINGLIST *
 rc_config_load(const char *file)
 {
@@ -239,6 +345,13 @@ rc_config_load(const char *file)
 	}
 	rc_stringlist_free(list);
 
+#ifdef __linux__
+	/* Only override rc.conf settings */
+	if (strcmp(file, RC_CONF) == 0) {
+		config = rc_config_override(config);
+	}
+#endif
+
 	return config;
 }
 librc_hidden_def(rc_config_load)
@@ -279,10 +392,13 @@ rc_conf_value(const char *setting)
 		atexit(_free_rc_conf);
 #endif
 
-		/* Support old configs */
+		/* Support old configs, but complain about it. */
 		if (exists(RC_CONF_OLD)) {
 			old = rc_config_load(RC_CONF_OLD);
 			TAILQ_CONCAT(rc_conf, old, entries);
+			ewarn("Your system still has %s", RC_CONF_OLD);
+			ewarn("Please migrate to the appropriate settings in %s", RC_CONF);
+			ewarn("and delete %s.", RC_CONF_OLD);
 #ifdef DEBUG_MEMORY
 			free(old);
 #endif
